@@ -1,6 +1,8 @@
 // Copyright Timothy H. Keitt 2015
 // See license for odeintr package
 
+// [[Rcpp::depends(odeintr)]]
+
 #include <Rcpp.h>
 // [[Rcpp::plugins(cpp11)]]
 
@@ -14,18 +16,14 @@ namespace odeintr
 {
   static const std::size_t N = __SYS_SIZE__;
   
-//  using state_type = std::array<double, N>;
-  typedef std::array<double, N> state_type;
+  typedef std::vector<double> state_type;
   
-  static state_type state;
+  static state_type state(N);
   
-//  using stepper_type = odeint::__STEPPER_TYPE__;
   typedef odeint::__STEPPER_TYPE__ stepper_type;
   
   static auto stepper = __STEPPER_CONSTRUCT__;
   
-//  using vec_type = Rcpp::List;
-//  using rec_type = std::vector<vec_type>;
   typedef Rcpp::List vec_type;
   typedef std::vector<vec_type> rec_type;
   static rec_type rec_x;
@@ -35,6 +33,8 @@ namespace odeintr
   
   __GLOBALS__;
   
+  #include "utils.h"
+  
   static void
   sys(const state_type x, state_type &dxdt, const double t)
   {
@@ -84,7 +84,7 @@ void __FUNCNAME___set_state(Rcpp::NumericVector new_state)
 std::vector<double>
 __FUNCNAME___get_state()
 {
-  return std::vector<double>(odeintr::state.begin(), odeintr::state.end());
+  return odeintr::state;
 }
 
 // [[Rcpp::export]]
